<?xml version="1.0"?><!--showtree.xsl-->
<?xml-stylesheet type="text/xsl" href="../xslstyle/xslstyle-docbook.xsl"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY nl "&#xa;"><!--new line sequence-->
<!--
  Rename this entity to "namespace-reporting" in order to disengage the
  inspection of the namespace axis, which will be important for those
  processors that trigger static errors for the presence of an XPath
  expression addressing the namespace axis.
-->
<!ENTITY namespace-reporting-rename-this-to-override-definition ''>

<!--the following declaration will only be used if not previously defined-->
<!ENTITY namespace-reporting '
    <xsl:if test="not($omit-namespaces)">
      <xsl:for-each select="namespace::*">
        <xsl:call-template name="namespace">
          <xsl:with-param name="ancestry" select="$this-ancestry"/>
          <xsl:with-param name="node-pos" select="$this-node-pos"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:if>
'>
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                exclude-result-prefixes="xs"
                version="1.0">

<xs:doc filename="showtree.xsl"
info="$Id: showtree.xsl,v 1.12 2009/05/01 13:29:01 gkholman Exp $"
        vocabulary="DocBook">
  <xs:title>SHOWTREE: XPath tree exposition stylesheet</xs:title>
  <para>
    This stylesheet walks all of the nodes of the XPath source tree, reporting
    the presence of each node and the value of all leaf nodes.
  </para>
  <para>
    This stylesheet relies on an entity with an address using the 
    namespace axis.  Since this is deprecated in the specification, there
    may be processors reporting static errors because of its presence.  To
    remove this use with the least amount of change, simply rename the entity 
    <literal>namespace-reporting-rename-this-to-override-definition</literal> 
    to be <literal>namespace-reporting</literal> and its empty definition will
    override the subsequent non-empty definition, thus prevent it from
    being used in the stylesheet.
  </para>
  <para>
    It also appears that the Microsoft XSLT processor in IE requires
    that the entity be empty, and as such namespaces cannot be reported.
  </para>
</xs:doc>

<xs:output>
  <para>The report is a simple text report</para>
</xs:output>
<xsl:output method="text"/>

<xs:param ignore-ns="yes">
  <para>
    Set this to <literal>true()</literal> to reveal the calculated value
    for each element.
  </para>
</xs:param>
<xsl:param name="show-element-values" select="false()"/>

<xs:param ignore-ns="yes">
  <para>
    Set this to <literal>true()</literal> to omit the reporting of
    namespaces.
  </para>
  <para>
    This relies on the <literal>namespace-reporting</literal> entity to
    be defined with the code that accesses the namespace axis.
  </para>
</xs:param>
<xsl:param name="omit-namespaces" select="false()"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Nodes with children</xs:title>
</xs:doc>

<xs:template>
  <para>Titling information and start of the tree descent.</para>
</xs:template>
<xsl:template match="/">                                       <!--root node-->
  <xsl:text>SHOWTREE - </xsl:text>
  <xsl:text>http://www.CraneSoftwrights.com/resources/</xsl:text>
  <xsl:text>&nl;Processor: </xsl:text>
  <xsl:value-of select="system-property('xsl:vendor')"/>
  <xsl:apply-templates select="node()"/>
  <xsl:text>
</xsl:text>
</xsl:template>

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
</xs:template>
<xsl:template match="*">                                      <!--an element-->
  <xsl:param name="ancestry"/>
  <xsl:param name="node-pos"/>
  <!--what are the parameters for this element?-->
  <xsl:variable name="this-ancestry">
    <xsl:if test="$ancestry!=''">
      <xsl:value-of select="$ancestry"/><xsl:text>,</xsl:text>
    </xsl:if>
    <xsl:value-of select="name(.)"/>
  </xsl:variable>
  <xsl:variable name="this-node-pos">
    <xsl:if test="$node-pos!=''">
      <xsl:value-of select="$node-pos"/><xsl:text>.</xsl:text>
    </xsl:if>
    <xsl:value-of select="position()"/>
  </xsl:variable>
  <!--report this element-->
  <xsl:call-template name="node">
    <xsl:with-param name="ancestry" select="$ancestry"/>
    <xsl:with-param name="node-pos" select="$this-node-pos"/>
    <xsl:with-param name="type">Element</xsl:with-param>
    <xsl:with-param name="contents">                          <!--node value-->
      <xsl:choose>
        <xsl:when test="not($show-element-values)"/>
        <xsl:when test="node()">
          <xsl:text>{</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>}</xsl:text>
        </xsl:when>
        <xsl:otherwise>no child nodes</xsl:otherwise>
      </xsl:choose>
      &namespace-reporting;<!--defined in a general entity in stylesheet DTD-->
      <xsl:for-each select="@*">
        <xsl:call-template name="attribute">
          <xsl:with-param name="ancestry" select="$this-ancestry"/>
          <xsl:with-param name="node-pos" select="$this-node-pos"/>
        </xsl:call-template>
      </xsl:for-each>
      <xsl:choose>                                         <!--node children-->
        <xsl:when test="node()">
          <xsl:apply-templates select="node()">
            <xsl:with-param name="ancestry" select="$this-ancestry"/>
            <xsl:with-param name="node-pos" select="$this-node-pos"/>
          </xsl:apply-templates>
        </xsl:when>
        <xsl:when test="$show-element-values"/>    <!--content already shown-->
      </xsl:choose>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>Leaf nodes</xs:title>
</xs:doc>

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
  <para>Report information regarding a comment node</para>
</xs:template>
<xsl:template match="comment()">                               <!--a comment-->
  <xsl:param name="ancestry"/>
  <xsl:param name="node-pos"/>
  <xsl:variable name="this-node-pos">
    <xsl:if test="$node-pos!=''">
      <xsl:value-of select="$node-pos"/><xsl:text>.</xsl:text>
    </xsl:if>
    <xsl:value-of select="position()"/>
  </xsl:variable>
  <xsl:call-template name="node">
    <xsl:with-param name="ancestry" select="$ancestry"/>
    <xsl:with-param name="node-pos" select="$this-node-pos"/>
    <xsl:with-param name="type">Comment</xsl:with-param>
    <xsl:with-param name="contents">
      <xsl:text>{</xsl:text><xsl:value-of select="."/><xsl:text>}</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
  <para>Report information regarding a processing node</para>
</xs:template>
<xsl:template match="processing-instruction()">       <!--a processing inst.-->
  <xsl:param name="ancestry"/>
  <xsl:param name="node-pos"/>
  <xsl:variable name="this-node-pos">
    <xsl:if test="$node-pos!=''">
      <xsl:value-of select="$node-pos"/><xsl:text>.</xsl:text>
    </xsl:if>
    <xsl:value-of select="position()"/>
  </xsl:variable>
  <xsl:call-template name="node">
    <xsl:with-param name="ancestry" select="$ancestry"/>
    <xsl:with-param name="node-pos" select="$this-node-pos"/>
    <xsl:with-param name="type">Proc. Inst.</xsl:with-param>
    <xsl:with-param name="contents">
      <xsl:text>{</xsl:text><xsl:value-of select="."/><xsl:text>}</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
  <para>Report information regarding a text node</para>
</xs:template>
<xsl:template match="text()">                                <!--a text node-->
  <xsl:param name="ancestry"/>
  <xsl:param name="node-pos"/>
  <xsl:variable name="this-node-pos">
    <xsl:value-of select="$node-pos"/><xsl:text>.</xsl:text>
    <xsl:value-of select="position()"/>
  </xsl:variable>
  <xsl:call-template name="node">
    <xsl:with-param name="ancestry" select="$ancestry"/>
    <xsl:with-param name="node-pos" select="$this-node-pos"/>
    <xsl:with-param name="type">Text</xsl:with-param>
    <xsl:with-param name="contents">
      <xsl:text>{</xsl:text><xsl:value-of select="."/><xsl:text>}</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!--attached leaf nodes-->

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
  <para>Report information regarding an attribute node</para>
</xs:template>
<xsl:template name="attribute">                             <!--an attribute-->
  <xsl:param name="node-pos"/>
  <xsl:variable name="ancestry"/><!--change to xsl:param to include ancestry-->
  <xsl:variable name="this-node-pos">
    <xsl:value-of select="$node-pos"/><xsl:text>.</xsl:text>
    <xsl:number value="position()" format="A"/>
  </xsl:variable>
  <xsl:call-template name="node">
    <xsl:with-param name="ancestry" select="$ancestry"/>
    <xsl:with-param name="node-pos" select="$this-node-pos"/>
    <xsl:with-param name="type">Attribute</xsl:with-param>
    <xsl:with-param name="contents">
      <xsl:text>{</xsl:text><xsl:value-of select="."/><xsl:text>}</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
  <para>Report information regarding a namespace node</para>
</xs:template>
<xsl:template name="namespace">                        <!--a namespace node-->
  <xsl:param name="node-pos"/>
  <xsl:variable name="ancestry"/><!--change to xsl:param to include ancestry-->
  <xsl:variable name="this-node-pos">
    <xsl:value-of select="$node-pos"/><xsl:text>.</xsl:text>
    <xsl:number value="position()" format="I"/>
  </xsl:variable>
  <xsl:call-template name="node">
    <xsl:with-param name="ancestry" select="$ancestry"/>
    <xsl:with-param name="node-pos" select="$this-node-pos"/>
    <xsl:with-param name="type">Namespace</xsl:with-param>
    <xsl:with-param name="contents">
      <xsl:text>{</xsl:text><xsl:value-of select="."/><xsl:text>}</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>Node information rendering</xs:title>
</xs:doc>

<xs:template>
  <xs:param name="ancestry">
    <para>The running calculated XPath string of the ancestry.</para>
  </xs:param>
  <xs:param name="node-pos">
    <para>The runing calculated ordinals string of the ancestry.</para>
  </xs:param>
  <xs:param name="type">
    <para>The kind of the node being reported.</para>
  </xs:param>
  <xs:param name="contents">
    <para>The content of the node being reported.</para>
  </xs:param>
  <para>
    For consistency, this one template rule does all of the actual reporting
    of information on each line.
  </para>
</xs:template>
<xsl:template name="node">
  <xsl:param name="ancestry"/>
  <xsl:param name="node-pos"/>
  <xsl:param name="type"/>
  <xsl:param name="contents"/>
  <xsl:text>&nl;</xsl:text>
  <xsl:value-of select="$node-pos"/>                     <!--report location-->
  <xsl:text>  </xsl:text>
  <xsl:value-of select="$type"/>                               <!--node type-->
  <xsl:if test="name(.)">                           <!--node name (if appl.)-->
    <xsl:text> '</xsl:text>
    <xsl:if test="namespace-uri(.)">                      <!--with namespace-->
      <xsl:text>{</xsl:text>
      <xsl:value-of select="namespace-uri(.)"/>
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:value-of select="name(.)"/>
    <xsl:text>'</xsl:text>
  </xsl:if>
                                                            <!--node context-->
  <xsl:if test="$ancestry"> (<xsl:value-of select="$ancestry"/>)</xsl:if>
  <xsl:text>: </xsl:text>
  <xsl:value-of select="$contents"/>
</xsl:template>

</xsl:stylesheet>